@@ -7,7 +7,7 @@ def parse_choice(env_str, default, arg_bool=False, arg_int=False):
         if arg_bool:
             if env_value is True or env_value is False:
                 return env_value
-            elif env_value.lower() in ["t", "true"]:
+            elif env_value.lower() in ["t", "true", "1", "y", "yes"]:
                 return True
             else:
                 return False
